#define _XOPEN_SOURCE
#include <stdio.h>
#include <unistd.h>
#include <signal.h>
#include <stdlib.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <dirent.h>
#include <string.h>
#include <crypt.h>


typedef void (*Handler)(int);
typedef unsigned char BOOLEAN;
struct our_dirent
{
  unsigned int    our_d_ino;
  unsigned int    our_d_off;
  unsigned int    our_d_reclen;
  unsigned char   our_d_name[256];
};

Handler Unix_Ignore = SIG_IGN;
Handler Unix_Default = SIG_DFL;

/*
Nachdem wir endlich wissen, wie wir das mit den Signalen machen
mssen, gleich eine berreaktion: alle Signale bereitstellen.
*/
int Unix_SIGHUP =       SIGHUP;    /* Hangup detected on controlling terminal
                                       or death of controlling process */
int Unix_SIGINT =       SIGINT;    /* Interrupt from keyboard */
int Unix_SIGQUIT =      SIGQUIT;   /* Quit from keyboard */
int Unix_SIGILL =       SIGILL;    /* Illegal Instruction */
int Unix_SIGABRT =      SIGABRT;   /* Abort signal from abort(3) */
int Unix_SIGFPE =       SIGFPE;    /* Floating point exception */
int Unix_SIGKILL =      SIGKILL;   /* Kill signal */
int Unix_SIGSEGV =      SIGSEGV;   /* Invalid memory reference */
int Unix_SIGPIPE =      SIGPIPE;   /* Broken pipe: write to pipe with no readers */
int Unix_SIGALRM =      SIGALRM;   /* Timer signal from alarm(1) */
int Unix_SIGTERM =      SIGTERM;   /* Termination signal */
int Unix_SIGUSR1 =      SIGUSR1;   /* User-defined signal 1 */
int Unix_SIGUSR2 =      SIGUSR2;   /* User-defined signal 2 */
int Unix_SIGCHLD =      SIGCHLD;   /* Child stopped or terminated */
int Unix_SIGCONT =      SIGCONT;   /* Continue if stopped */
int Unix_SIGSTOP =      SIGSTOP;   /* Stop process */
int Unix_SIGTSTP =      SIGTSTP;   /* Stop typed at tty */
int Unix_SIGTTIN =      SIGTTIN;   /* tty input for background process */
int Unix_SIGTTOU =      SIGTTOU;   /* tty output for background process */

int Unix_SIGTRAP =      SIGTRAP;   /* Trace/breakpoint trap */
int Unix_SIGIOT =       SIGIOT;    /* IOT trap. A synonym for SIGABRT */
int Unix_SIGBUS =       SIGBUS;    /* Bus error */
int Unix_SIGURG =       SIGURG;    /* Urgent condition on socket (4.2 BSD) */
int Unix_SIGIO =        SIGIO;     /* I/O now possible (4.2 BSD) */
int Unix_SIGXCPU =      SIGXCPU;   /* CPU time limit exceeded (4.2 BSD) */
int Unix_SIGXFSZ =      SIGXFSZ;   /* File size limit exceeded (4.2 BSD) */
int Unix_SIGVTALRM =    SIGVTALRM; /* Virtual alarm clock (4.2 BSD) */
int Unix_SIGPROF =      SIGPROF;   /* Profile alarm clock */
int Unix_SIGWINCH =     SIGWINCH;  /* Window resize signal (4.3 BSD, Sun) */

struct our_dirent               unser_dirent;

unsigned char *Unix_Crypt(unsigned char *key, unsigned char *salt)
  {
    return crypt(key, salt);
  }


BOOLEAN Unix_InstallHandler(int sigNum, Handler proc, Handler *old)
  {
    /* Assert: old != NULL */
    *old = signal(sigNum, proc);
    return *old != SIG_ERR;
  }


BOOLEAN Unix_RaiseSignal(int sigNum)
  {
    int                   rueckgabe;

    rueckgabe = raise(sigNum);
    if (rueckgabe == 0)
      {
        return 1;
      }
    else
      {
        return 0;
      }
  }


void Unix_Atexit(void (*p)(void))
  {
    (void) atexit(p); /* discard int return value */
  }


long Unix_MkTime(struct tm *timeptr)
  {
    long tmp;

    tmp = mktime(timeptr);

    return tmp;
  }


struct tm *Unix_GmTime(unsigned int time)
  {
    struct tm *tmp;
    time_t ltime;

    ltime = time;
    tmp = gmtime(&ltime);

    return tmp;
  }


struct tm *Unix_LocalTime(unsigned int time)
  {
    struct tm *tmp;
    time_t ltime;

    ltime = time;
    tmp = localtime(&ltime);

    return tmp;
  }


BOOLEAN Unix_Link(unsigned char *altpfad, unsigned char *neupfad)
  {
    int                   rueckgabe;

    rueckgabe = link(altpfad, neupfad);
    if (rueckgabe == 0)
      {
        return 1;
      }
    else
      {
        return 0;
      }
  }


BOOLEAN Unix_Unlink(unsigned char *pfad)
  {
    int                   rueckgabe;

    rueckgabe = unlink(pfad);
    if (rueckgabe == 0)
      {
        return 1;
      }
    else
      {
        return 0;
      }
  }


DIR *Unix_Opendir(unsigned char *name)
  {
    return opendir(name);
  }


BOOLEAN Unix_Closedir(DIR *dirp)
  {
    int                   rueckgabe;

    rueckgabe = closedir(dirp);
    if (rueckgabe == 0)
      {
        return 1;
      }
    else
      {
        return 0;
      }
  }


struct our_dirent *Unix_Readdir(DIR *dirp)
  {
    struct dirent               *rueckgabe;

    rueckgabe = readdir(dirp);
    if (rueckgabe == NULL)
      {
        return NULL;
      }
    else
      {
        unser_dirent.our_d_ino = (*(rueckgabe)).d_ino;
        /*
        unser_dirent.our_d_off = (*(rueckgabe)).d_off;
        */
        unser_dirent.our_d_off = 0;
        unser_dirent.our_d_reclen = (*(rueckgabe)).d_reclen;
        strcpy(unser_dirent.our_d_name, (*(rueckgabe)).d_name);
        return &unser_dirent;
      }
  }


int Unix_Mkdir(unsigned char *name, mode_t mode)
  {
    return mkdir(name, mode);
  }


int Unix_Rmdir(unsigned char *name)
  {
    return rmdir(name);
  }


int Unix_System(unsigned char *name)
  {
    return system(name);
  }


void Unix_GetEnv(char * str1, char * str2, unsigned int h2)
  {
    char *                      strptr;

    // printf("h2: %d\n", h2);
    // printf("str1: %s, str2: %s\n", str1, str2);

    strptr = getenv(str1);
    if (strptr != NULL)
    {
      // printf("strptr != NULL\n");
      // printf("strlen(strptr): %d\n", strlen(strptr));
      if (strlen(strptr) > h2)
      {
        fprintf(stderr, "Wert der Env-Variable %s ist zu lang\n", str1);
        exit(1);
      }
      else
      {
        strcpy(str2, strptr);
      }
    }
    else
    {
      // printf("strptr == NULL\n");
      str2[0] = 0;
    }
    return;
  }


unsigned int Unix_Time(void)
  {
    time_t zeit;

    zeit = time(NULL);

    return (unsigned int)zeit;
  }


// ein bisschen gespickt bei stat(2)
void Unix_ModifyTime(unsigned * zeit, char * ok, char * pfad)
  {
    int ret_val_stat;
    int tmp;
    struct stat sb;

    // printf("pfad: %s\n", pfad);
    // printf("(*zeit): %d\n", (*zeit));
    tmp = (*ok);
    // printf("BOOLEAN: %d\n", tmp);

    ret_val_stat = stat(pfad, &sb);
    if (ret_val_stat != 0)
    {
      // printf("ret_val_stat != 0\n");
      (*ok) = 0;
      return;
    }
    else
    {
      // printf("ret_val_stat == 0\n");
      (*zeit) = sb.st_mtime;
      (*ok) = 1;
      return;
    }
  }


unsigned int Unix_Sleep(unsigned int seconds)
  {
    return sleep(seconds);
  }


/*
for4.c/21.05.1998:
        Neben MeinCrypt werden wir nun auch was mit Signalen machen.

for4.c/24.05.1998:
        atexit kommt auch wieder mit ins Boot. Auerdem auch raise.

for4/30.05.1998:
        mktime testen.

for4/31.05.1998:
        Cont.

unix/14.06.1998:
        Endlich richtige Namensgebung im Kommentar. Die Zuweisungen der
        Form: timezone = Unix_Timezone vor Aufruf von mktime etc. lassen
        wir wieder weg.

unix/13.12.1998:
        link.

unix/03.03.1999:
        Heute werfen wir 4 linux-spezifische Signale raus, damit auf
        freeBSD auch alles glatt geht. Auerdem brauchen wir auch
        daylight, timezone und *tzname nicht.

unix/21.03.1999:
        Wir wollen Directories selber lesen. Dazu brauchen wir
        wenigstens 'opendir', 'readdir' und 'closedir' von der libc.

unix/18.11.1999:
        Wir wollen Directories auch selber anlegen. Dazu brauchen
        wir mkdir (und zum Spa rmdir).

unix/23.01.2001:
        'system' aufnehmen, um einfach Infomails verschicken zu
        knnen.

unix/29.03.2023:
        getenv, time.

unix/30.03.2023:
        stat / 'modifytime'.
        unlink.

Unix/31.03.2023:
        sleep.

Unix/02.04.2023:
        Ein paar Tests bezglich Parameterbergabe bei GM2.

Unix/04.04.2023:
        Rckport auf GPM.

Unix/05.04.2023:
        Ein bisschen debuggen.

Unix/23.05.2023:
        Dies ist die Version fr GM2.
        Bis auf den einen oder anderen Kommentar und Unix_GetEnv() sollten
        die Versionen fr GPM und GM2 ident sein.

Unix/07.12.2023:
        Paramter-Tests wieder rausschmeien.
*/
